#
# Abstraction layer for VFS operations to move gnomevfs dependency out of the
# core.
#

if (HAVE_WIN32):
    import urllib2

    OPEN_READ = "r"
    OPEN_WRITE = "w"

else:
    try:
        import gnomevfs
    except ImportError:
        import gnome.vfs as gnomevfs

    OPEN_READ = gnomevfs.OPEN_READ
    OPEN_WRITE = gnomevfs.OPEN_WRITE


#
# Reads the entire file and returns its contents.
#
def read_entire_file(uri):

    try:
        uri = gnomevfs.read_entire_file(uri)
    except:
        log("Warning: Couldn't read file \"%s\"." % (uri,))
        raise

    return uri

def read_entire_file_win32(uri):

    try:
        uri = urllib2.urlopen(uri).read()
    except:
        log("Warning: Couldn't read file \"%s\"." % (uri,))
        raise

    return uri



#
# Opens the given URI and returns a file descriptor.
#
def open(uri, mode = OPEN_READ):

    try:
        fd = gnomevfs.open(uri, mode)
    except:
        log("Warning: Couldn't open file \"%s\"." % (uri,))
        return

    return fd

def open_win32(uri, mode = OPEN_READ):

    try:
        fd = urllib2.urlopen(uri)
    except:
        log("Warning: Couldn't open file \"%s\"." % (uri,))
        return

    return fd



#
# Returns whether the given URI exists.
#
def exists(uri):

    return gnomevfs.exists(gnomevfs.URI(uri))

def exists_win32(uri):

    try:
        urllib2.urlopen(uri)
    except:
        return False
    else:
        return True


if (HAVE_WIN32):
    read_entire_file = read_entire_file_win32
    open = open_win32
    exists = exists_win32
